"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const flat = require("flat");
const util_1 = require("util");
const readdir = util_1.promisify(fs.readdir);
const lstat = util_1.promisify(fs.lstat);
const exists = util_1.promisify(fs.exists);
const readFile = util_1.promisify(fs.readFile);
function mapAsync(array, callbackfn) {
    return Promise.all(array.map(callbackfn));
}
async function filterAsync(array, callbackfn) {
    const filterMap = await mapAsync(array, callbackfn);
    return array.filter((value, index) => filterMap[index]);
}
const isDirectory = async (source) => (await lstat(source)).isDirectory();
exports.getDirectories = async (source) => {
    const dirs = await readdir(source);
    return filterAsync(dirs.map(name => path.join(source, name)), isDirectory);
};
const getFiles = async (dirPath, pattern) => {
    const dirs = await readdir(dirPath, { withFileTypes: true });
    return (await filterAsync(dirs, async (f) => {
        try {
            if (typeof f === 'string') {
                return (await exists(path.join(dirPath, f))) && pattern.test(f);
            }
            else {
                return f.isFile() && pattern.test(f.name);
            }
        }
        catch (_a) {
            return false;
        }
    })).map(f => path.join(dirPath, typeof f === 'string' ? f : f.name));
};
async function getLanguages(options) {
    const i18nPath = path.normalize(options.path + path.sep);
    return (await exports.getDirectories(i18nPath)).map(dir => path.relative(i18nPath, dir));
}
exports.getLanguages = getLanguages;
async function parseTranslations(options) {
    const i18nPath = path.normalize(options.path + path.sep);
    const translations = {};
    if (!(await exists(i18nPath))) {
        throw new Error(`i18n path (${i18nPath}) cannot be found`);
    }
    if (!options.filePattern.match(/\*\.[A-z]+/)) {
        throw new Error(`filePattern should be formatted like: *.json, *.txt, *.custom etc`);
    }
    const languages = await getLanguages(options);
    const pattern = new RegExp('.' + options.filePattern.replace('.', '.'));
    const files = await [
        ...languages.map(l => path.join(i18nPath, l)),
        i18nPath,
    ].reduce(async (files, path) => {
        (await files).push(...(await getFiles(path, pattern)));
        return files;
    }, Promise.resolve([]));
    for (const file of files) {
        let global = false;
        const key = path.dirname(path.relative(i18nPath, file)).split(path.sep)[0];
        if (key === '.') {
            global = true;
        }
        const data = JSON.parse(await readFile(file, 'utf8'));
        const prefix = path.basename(file).split('.')[0];
        const flatData = flat.flatten(data);
        for (const property of Object.keys(flatData)) {
            [...(global ? languages : [key])].forEach(lang => {
                translations[lang] = !!translations[lang] ? translations[lang] : {};
                translations[lang][`${global ? '' : `${prefix}.`}${property}`] =
                    flatData[property];
            });
        }
    }
    return translations;
}
exports.parseTranslations = parseTranslations;
